"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ApolloCacheVisitor = exports.validate = exports.plugin = void 0;
const path_1 = require("path");
const graphql_1 = require("graphql");
const cacheVisitor_1 = require("./cacheVisitor");
Object.defineProperty(exports, "ApolloCacheVisitor", { enumerable: true, get: function () { return cacheVisitor_1.ApolloCacheVisitor; } });
const plugin = (schema, documents, config) => {
    const allAst = graphql_1.concatAST(documents.map((v) => v.document));
    const allFragments = [
        ...allAst.definitions.filter((d) => d.kind === graphql_1.Kind.FRAGMENT_DEFINITION).map((fragmentDef) => ({
            node: fragmentDef,
            name: fragmentDef.name.value,
            onType: fragmentDef.typeCondition.name.value,
            isExternal: false,
        })),
        ...(config.externalFragments || []),
    ];
    const visitor = new cacheVisitor_1.ApolloCacheVisitor(schema, allFragments, config, documents);
    let visitorResult = graphql_1.visit(allAst, { leave: visitor });
    const fragments = visitor.buildOperationReadFragmentCache();
    return {
        prepend: visitor.getImports(),
        content: [
            ...visitorResult.definitions.filter((t) => typeof t === "string"),
            fragments,
        ].join("\n"),
    };
};
exports.plugin = plugin;
const validate = (schema, documents, config, outputFile) => __awaiter(void 0, void 0, void 0, function* () {
    if (path_1.extname(outputFile) !== ".tsx") {
        throw new Error(`Plugin "react-apollo" requires extension to be ".tsx"!`);
    }
});
exports.validate = validate;
